from typing import Type, Any, List
from nexios.http import Response, Request
from tortoise.models import Model
from tortoise.queryset import QuerySet
from pydantic import BaseModel

class CreateModelMixin:
    async def create(self, req: Request, res: Response) -> Response:
        try:
            instance = await self.validate_request()
        except ValueError as e:
            return self.handle_validation_error(res, e)

        try:
            db_instance = await self.perform_create(instance)
        except Exception as e:
            return self.handle_create_error(res, e)

        return self.handle_create_success(res, db_instance)

    async def perform_create(self, instance: BaseModel) -> Model:
        return await self.queryset.create(**instance.dict())

    def handle_validation_error(self, res: Response, error: ValueError) -> Response:
        return res.status(400).json(self.format_error_response("Validation failed", details=error.args[0]))

    def handle_create_error(self, res: Response, error: Exception) -> Response:
        return res.status(500).json(self.format_error_response("Failed to create object", details=str(error)))

    def handle_create_success(self, res: Response, instance: Model) -> Response:
        return res.status(201).json(self.format_success_response(instance.dict()))

class RetrieveModelMixin:
    async def retrieve(self, req: Request, res: Response) -> Response:
        try:
            obj = await self.get_object()
        except ValueError as e:
            return self.handle_not_found_error(res, e)

        serialized_obj = await self.serialize_object(obj)
        return self.handle_retrieve_success(res, serialized_obj)

    async def serialize_object(self, obj: Model) -> dict:
        pydantic_class = self.get_pydantic_class()
        return pydantic_class.from_orm(obj).dict()

    def handle_not_found_error(self, res: Response, error: ValueError) -> Response:
        return res.status(404).json(self.format_error_response(str(error)))

    def handle_retrieve_success(self, res: Response, serialized_obj: dict) -> Response:
        return res.status(200).json(self.format_success_response(serialized_obj))

class ListModelMixin:
    async def list(self, req: Request, res: Response) -> Response:
        queryset = await self.get_queryset()
        try:
            objects = await self.perform_list(queryset)
        except Exception as e:
            return self.handle_list_error(res, e)

        serialized_objects = await self.serialize_objects(objects)
        return self.handle_list_success(res, serialized_objects)

    async def perform_list(self, queryset: QuerySet) -> List[Model]:
        return await queryset.all()

    async def serialize_objects(self, objects: List[Model]) -> List[dict]:
        pydantic_class = self.get_pydantic_class()
        return [pydantic_class.from_orm(obj).dict() for obj in objects]

    def handle_list_error(self, res: Response, error: Exception) -> Response:
        raise error
    def handle_list_success(self, res: Response, serialized_objects: List[dict]) -> Response:
        return res.status(200).json(self.format_success_response(serialized_objects))

class DeleteModelMixin:
    async def delete(self, req: Request, res: Response) -> Response:
        try:
            obj = await self.get_object()
        except ValueError as e:
            return self.handle_not_found_error(res, e)

        try:
            await self.perform_delete(obj)
        except Exception as e:
            return self.handle_delete_error(res, e)

        return self.handle_delete_success(res)

    async def perform_delete(self, obj: Model) -> None:
        await obj.delete()

    def handle_delete_error(self, res: Response, error: Exception) -> Response:
        raise error

    def handle_delete_success(self, res: Response) -> Response:
        return res.status(204).json(self.format_success_response(None, status_code=204))

class UpdateModelMixin:
    async def update(self, req: Request, res: Response) -> Response:
        try:
            obj = await self.get_object()
        except ValueError as e:
            return self.handle_not_found_error(res, e)

        try:
            instance = await self.validate_request()
        except ValueError as e:
            return self.handle_validation_error(res, e)

        try:
            updated_obj = await self.perform_update(obj, instance)
        except Exception as e:
            return self.handle_update_error(res, e)

        serialized_obj = await self.serialize_object(updated_obj)
        return self.handle_update_success(res, serialized_obj)

    async def perform_update(self, obj: Model, instance: BaseModel) -> Model:
        for key, value in instance.dict().items():
            setattr(obj, key, value)
        await obj.save()
        return obj

    def handle_update_error(self, res: Response, error: Exception) -> Response:
        return res.status(500).json(self.format_error_response("Failed to update object", details=str(error)))

    def handle_update_success(self, res: Response, serialized_obj: dict) -> Response:
        return res.status(200).json(self.format_success_response(serialized_obj))


from typing import Type, Any, Dict, List, Optional, Generic, TypeVar
from pydantic import BaseModel, ValidationError
from tortoise.queryset import QuerySet
from tortoise.models import Model
from nexios.views import APIView

T = TypeVar('T', bound=Model)
P = TypeVar('P', bound=BaseModel)

class GenericAPIView(APIView, Generic[T, P]):
    queryset: QuerySet[T] = None
    pydantic_class: Type[P] = None
    lookup_field: str = 'pk'
    lookup_url_kwarg: str = None

    async def get_queryset(self) -> QuerySet[T]:
        assert self.queryset is not None, (
            "'%s' should either include a `queryset` attribute, "
            "or override the `get_queryset()` method."
            % self.__class__.__name__
        )
        if isinstance(self.queryset, QuerySet):
            queryset = self.queryset.all()
        else:
            queryset = self.queryset
        return queryset

    def get_pydantic_class(self) -> Type[P]:
        assert self.pydantic_class is not None, (
            "'%s' should either include a `pydantic_class` attribute, "
            "or override the `get_pydantic_class()` method."
            % self.__class__.__name__
        )
        return self.pydantic_class

    async def validate_request(self) -> P:
        pydantic_class = self.get_pydantic_class()
        data = await self.request.json()
        try:
            instance = pydantic_class(**data)
        except ValidationError as e:
            raise ValueError({"errors": e.errors()})
        return instance

    async def get_object(self) -> T:
        queryset = await self.get_queryset()
        lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field
        filter_kwargs = {self.lookup_field: self.request.path_params[lookup_url_kwarg]}
        try:
            obj = await queryset.get(**filter_kwargs)
        except Exception as e:
            raise ValueError(f"Object not found: {e}")
        return obj

    def format_error_response(self, message: str, status_code: int = 400, details: Optional[Dict] = None) -> Dict:
        return {
            "status": "error",
            "message": message,
            "code": status_code,
            "details": details or {}
        }

    def format_success_response(self, data: Any, status_code: int = 200) -> Dict:
        return {
            "status": "success",
            "code": status_code,
            "data": data
        }


from .base import GenericAPIView
from .mixins import ListModelMixin, CreateModelMixin, RetrieveModelMixin, UpdateModelMixin, DeleteModelMixin

class ListCreateAPIView(ListModelMixin, CreateModelMixin, GenericAPIView):
    async def get(self, req, res):
        return await self.list(req, res)
    
    async def post(self, req, res):
        return await self.create(req, res)

class RetrieveUpdateDestroyAPIView(RetrieveModelMixin, UpdateModelMixin, DeleteModelMixin, GenericAPIView):
    async def get(self, req, res):
        return await self.retrieve(req, res)
    
    async def put(self, req, res):
        return await self.update(req, res)
    
    async def delete(self, req, res):
        return await self.delete(req, res)

class ListCreateRetrieveUpdateDestroyAPIView(ListCreateAPIView, RetrieveUpdateDestroyAPIView):
    pass